@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2010 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -22,16 +22,15 @@
 //#define USE_MATRIX_TEST_DEBUG
 
 #if defined( USE_MATRIX_TEST_DEBUG )
-	#include <windows.h>
 
-	struct MatrixTest
-	{
-		MatrixTest () : ID(0), Calls(0) {}
-		char buf[256];
-		int Calls;
-		int ID;
-	};
-	static MatrixTest MTest;
+struct MatrixTest
+{
+	MatrixTest () : ID(0), Calls(0) {}
+	char buf[256];
+	int Calls;
+	int ID;
+};
+static MatrixTest MTest;
 
 #endif
 
@@ -64,11 +63,11 @@ namespace core
 			//! Copy constructor
 			/** \param other Other matrix to copy from
 			\param constructor Choose the initialization style */
-			CMatrix4( const CMatrix4<T>& other,eConstructor constructor = EM4CONST_COPY);
+			CMatrix4(const CMatrix4<T>& other, eConstructor constructor = EM4CONST_COPY);
 
 			//! Simple operator for directly accessing every element of the matrix.
 			T& operator()(const s32 row, const s32 col)
-			{ 
+			{
 #if defined ( USE_MATRIX_TEST )
 				definitelyIdentityMatrix=false;
 #endif
@@ -80,9 +79,9 @@ namespace core
 
 			//! Simple operator for linearly accessing every element of the matrix.
 			T& operator[](u32 index)
-			{ 
+			{
 #if defined ( USE_MATRIX_TEST )
-				definitelyIdentityMatrix=false; 
+				definitelyIdentityMatrix=false;
 #endif
 				return M[index];
 			}
@@ -98,8 +97,8 @@ namespace core
 
 			//! Returns pointer to internal array
 			const T* pointer() const { return M; }
-			T* pointer() 
-			{ 
+			T* pointer()
+			{
 #if defined ( USE_MATRIX_TEST )
 				definitelyIdentityMatrix=false;
 #endif
@@ -125,17 +124,20 @@ namespace core
 			CMatrix4<T>& operator-=(const CMatrix4<T>& other);
 
 			//! set this matrix to the product of two matrices
+			/** Calculate b*a */
 			inline CMatrix4<T>& setbyproduct(const CMatrix4<T>& other_a,const CMatrix4<T>& other_b );
 
 			//! Set this matrix to the product of two matrices
-			/** no optimization used,
+			/** Calculate b*a, no optimization used,
 			use it if you know you never have a identity matrix */
 			CMatrix4<T>& setbyproduct_nocheck(const CMatrix4<T>& other_a,const CMatrix4<T>& other_b );
 
 			//! Multiply by another matrix.
+			/** Calculate other*this */
 			CMatrix4<T> operator*(const CMatrix4<T>& other) const;
 
 			//! Multiply by another matrix.
+			/** Calculate and return other*this */
 			CMatrix4<T>& operator*=(const CMatrix4<T>& other);
 
 			//! Multiply by scalar.
@@ -181,7 +183,11 @@ namespace core
 
 			//! Make an inverted rotation matrix from Euler angles.
 			/** The 4th row and column are unmodified. */
-			CMatrix4<T>& setInverseRotationDegrees( const vector3d<T>& rotation );
+			inline CMatrix4<T>& setInverseRotationDegrees( const vector3d<T>& rotation );
+
+			//! Make a rotation matrix from angle and axis, assuming left handed rotation.
+			/** The 4th row and column are unmodified. */
+			inline CMatrix4<T>& setRotationAxisRadians(const T& angle, const vector3d<T>& axis);
 
 			//! Set Scale
 			CMatrix4<T>& setScale( const vector3d<T>& scale );
@@ -216,6 +222,9 @@ namespace core
 			//! An alternate transform vector method, writing into an array of 4 floats
 			void transformVect(T *out,const core::vector3df &in) const;
 
+			//! An alternate transform vector method, reading from and writing to an array of 3 floats
+			void transformVec3(T *out, const T * in) const;
+
 			//! Translate a vector by the translation part of this matrix.
 			void translateVect( vector3df& vect ) const;
 
@@ -258,6 +267,9 @@ namespace core
 			//! Builds a left-handed perspective projection matrix based on a field of view
 			CMatrix4<T>& buildProjectionMatrixPerspectiveFovLH(f32 fieldOfViewRadians, f32 aspectRatio, f32 zNear, f32 zFar);
 
+			//! Builds a left-handed perspective projection matrix based on a field of view, with far plane at infinity
+			CMatrix4<T>& buildProjectionMatrixPerspectiveFovInfinityLH(f32 fieldOfViewRadians, f32 aspectRatio, f32 zNear, f32 epsilon=0);
+
 			//! Builds a right-handed perspective projection matrix.
 			CMatrix4<T>& buildProjectionMatrixPerspectiveRH(f32 widthOfViewVolume, f32 heightOfViewVolume, f32 zNear, f32 zFar);
 
@@ -772,11 +784,11 @@ namespace core
 	}
 
 	//! Returns the absolute values of the scales of the matrix.
-	/** 
-	Note that this always returns the absolute (positive) values.  Unfortunately it
-	does not appear to be possible to extract any original negative values.  The best
-	that we could do would be to arbitrarily make one scale negative if one or three
-	of them were negative.
+	/**
+	Note that this returns the absolute (positive) values unless only scale is set.
+	Unfortunately it does not appear to be possible to extract any original negative
+	values. The best that we could do would be to arbitrarily make one scale
+	negative if one or three of them were negative.
 	FIXME - return the original values.
 	*/
 	template <class T>
@@ -848,10 +860,26 @@ namespace core
 	inline core::vector3d<T> CMatrix4<T>::getRotationDegrees() const
 	{
 		const CMatrix4<T> &mat = *this;
-		const core::vector3d<T> scale = getScale();
+		core::vector3d<T> scale = getScale();
+		// we need to check for negative scale on to axes, which would bring up wrong results
+		if (scale.Y<0 && scale.Z<0)
+		{
+			scale.Y =-scale.Y;
+			scale.Z =-scale.Z;
+		}
+		else if (scale.X<0 && scale.Z<0)
+		{
+			scale.X =-scale.X;
+			scale.Z =-scale.Z;
+		}
+		else if (scale.X<0 && scale.Y<0)
+		{
+			scale.X =-scale.X;
+			scale.Y =-scale.Y;
+		}
 		const core::vector3d<f64> invScale(core::reciprocal(scale.X),core::reciprocal(scale.Y),core::reciprocal(scale.Z));
 
-		f64 Y = -asin(mat[2]*invScale.X);
+		f64 Y = -asin(core::clamp(mat[2]*invScale.X, -1.0, 1.0));
 		const f64 C = cos(Y);
 		Y *= RADTODEG64;
 
@@ -876,8 +904,6 @@ namespace core
 		}
 
 		// fix values that get below zero
-		// before it would set (!) values to 360
-		// that were above 360:
 		if (X < 0.0) X += 360.0;
 		if (Y < 0.0) Y += 360.0;
 		if (Z < 0.0) Z += 360.0;
@@ -886,6 +912,7 @@ namespace core
 	}
 
 
+	//! Sets matrix to rotation matrix of inverse angles given as parameters
 	template <class T>
 	inline CMatrix4<T>& CMatrix4<T>::setInverseRotationRadians( const vector3d<T>& rotation )
 	{
@@ -916,6 +943,40 @@ namespace core
 		return *this;
 	}
 
+	//! Sets matrix to rotation matrix defined by axis and angle, assuming LH rotation
+	template <class T>
+	inline CMatrix4<T>& CMatrix4<T>::setRotationAxisRadians( const T& angle, const vector3d<T>& axis )
+	{
+ 		const f64 c = cos(angle);
+		const f64 s = sin(angle);
+		const f64 t = 1.0 - c;
+
+		const f64 tx  = t * axis.X;
+		const f64 ty  = t * axis.Y;		
+		const f64 tz  = t * axis.Z;
+
+		const f64 sx  = s * axis.X;
+		const f64 sy  = s * axis.Y;
+		const f64 sz  = s * axis.Z;
+		
+		M[0] = (T)(tx * axis.X + c);
+		M[1] = (T)(tx * axis.Y + sz);
+		M[2] = (T)(tx * axis.Z - sy);
+
+		M[4] = (T)(ty * axis.X - sz);
+		M[5] = (T)(ty * axis.Y + c);
+		M[6] = (T)(ty * axis.Z + sx);
+
+		M[8]  = (T)(tz * axis.X + sy);
+		M[9]  = (T)(tz * axis.Y - sx);
+		M[10] = (T)(tz * axis.Z + c);
+
+#if defined ( USE_MATRIX_TEST )
+		definitelyIdentityMatrix=false;
+#endif
+		return *this;
+	}
+
 
 	/*!
 	*/
@@ -942,17 +1003,29 @@ namespace core
 		if (definitelyIdentityMatrix)
 			return true;
 #endif
+		if (!core::equals( M[12], (T)0 ) || !core::equals( M[13], (T)0 ) || !core::equals( M[14], (T)0 ) || !core::equals( M[15], (T)1 ))
+			return false;
+
+		if (!core::equals( M[ 0], (T)1 ) || !core::equals( M[ 1], (T)0 ) || !core::equals( M[ 2], (T)0 ) || !core::equals( M[ 3], (T)0 ))
+			return false;
+
+		if (!core::equals( M[ 4], (T)0 ) || !core::equals( M[ 5], (T)1 ) || !core::equals( M[ 6], (T)0 ) || !core::equals( M[ 7], (T)0 ))
+			return false;
+
+		if (!core::equals( M[ 8], (T)0 ) || !core::equals( M[ 9], (T)0 ) || !core::equals( M[10], (T)1 ) || !core::equals( M[11], (T)0 ))
+			return false;
+/*
 		if (!core::equals( M[ 0], (T)1 ) ||
-				!core::equals( M[ 5], (T)1 ) ||
-				!core::equals( M[10], (T)1 ) ||
-				!core::equals( M[15], (T)1 ))
+			!core::equals( M[ 5], (T)1 ) ||
+			!core::equals( M[10], (T)1 ) ||
+			!core::equals( M[15], (T)1 ))
 			return false;
 
 		for (s32 i=0; i<4; ++i)
 			for (s32 j=0; j<4; ++j)
 				if ((j != i) && (!iszero((*this)(i,j))))
 					return false;
-
+*/
 #if defined ( USE_MATRIX_TEST )
 		definitelyIdentityMatrix=true;
 #endif
@@ -1092,6 +1165,14 @@ namespace core
 		out[3] = in.X*M[3] + in.Y*M[7] + in.Z*M[11] + M[15];
 	}
 
+	template <class T>
+	inline void CMatrix4<T>::transformVec3(T *out, const T * in) const
+	{
+		out[0] = in[0]*M[0] + in[1]*M[4] + in[2]*M[8] + M[12];
+		out[1] = in[0]*M[1] + in[1]*M[5] + in[2]*M[9] + M[13];
+		out[2] = in[0]*M[2] + in[1]*M[6] + in[2]*M[10] + M[14];
+	}
+
 
 	//! Transforms a plane by this matrix
 	template <class T>
@@ -1245,7 +1326,7 @@ namespace core
 			(m(0, 1) * m(1, 3) - m(0, 3) * m(1, 1)) * (m(2, 0) * m(3, 2) - m(2, 2) * m(3, 0)) +
 			(m(0, 2) * m(1, 3) - m(0, 3) * m(1, 2)) * (m(2, 0) * m(3, 1) - m(2, 1) * m(3, 0));
 
-		if( core::iszero ( d ) )
+		if( core::iszero ( d, FLT_MIN ) )
 			return false;
 
 		d = core::reciprocal ( d );
@@ -1413,7 +1494,7 @@ namespace core
 	{
 		const f64 h = reciprocal(tan(fieldOfViewRadians*0.5));
 		_IRR_DEBUG_BREAK_IF(aspectRatio==0.f); //divide by zero
-		const T w = h / aspectRatio;
+		const T w = static_cast<T>(h / aspectRatio);
 
 		_IRR_DEBUG_BREAK_IF(zNear==zFar); //divide by zero
 		M[0] = w;
@@ -1452,7 +1533,7 @@ namespace core
 	{
 		const f64 h = reciprocal(tan(fieldOfViewRadians*0.5));
 		_IRR_DEBUG_BREAK_IF(aspectRatio==0.f); //divide by zero
-		const T w = (T)(h / aspectRatio);
+		const T w = static_cast<T>(h / aspectRatio);
 
 		_IRR_DEBUG_BREAK_IF(zNear==zFar); //divide by zero
 		M[0] = w;
@@ -1482,6 +1563,42 @@ namespace core
 	}
 
 
+	// Builds a left-handed perspective projection matrix based on a field of view, with far plane culling at infinity
+	template <class T>
+	inline CMatrix4<T>& CMatrix4<T>::buildProjectionMatrixPerspectiveFovInfinityLH(
+			f32 fieldOfViewRadians, f32 aspectRatio, f32 zNear, f32 epsilon)
+	{
+		const f64 h = reciprocal(tan(fieldOfViewRadians*0.5));
+		_IRR_DEBUG_BREAK_IF(aspectRatio==0.f); //divide by zero
+		const T w = static_cast<T>(h / aspectRatio);
+
+		M[0] = w;
+		M[1] = 0;
+		M[2] = 0;
+		M[3] = 0;
+
+		M[4] = 0;
+		M[5] = (T)h;
+		M[6] = 0;
+		M[7] = 0;
+
+		M[8] = 0;
+		M[9] = 0;
+		M[10] = (T)(1.f-epsilon);
+		M[11] = 1;
+
+		M[12] = 0;
+		M[13] = 0;
+		M[14] = (T)(zNear*(epsilon-1.f));
+		M[15] = 0;
+
+#if defined ( USE_MATRIX_TEST )
+		definitelyIdentityMatrix=false;
+#endif
+		return *this;
+	}
+
+
 	// Builds a left-handed orthogonal projection matrix.
 	template <class T>
 	inline CMatrix4<T>& CMatrix4<T>::buildProjectionMatrixOrthoLH(
@@ -1817,22 +1934,22 @@ namespace core
 	inline CMatrix4<T>& CMatrix4<T>::buildRotateFromTo(const core::vector3df& from, const core::vector3df& to)
 	{
 		// unit vectors
-		core::vector3df f ( from );
-		core::vector3df t ( to );
-		f.normalize ();
-		t.normalize ();
+		core::vector3df f(from);
+		core::vector3df t(to);
+		f.normalize();
+		t.normalize();
 
 		// axis multiplication by sin
-		core::vector3df vs ( t.crossProduct ( f ) );
+		core::vector3df vs(t.crossProduct(f));
 
 		// axis of rotation
-		core::vector3df v ( vs );
+		core::vector3df v(vs);
 		v.normalize();
 
 		// cosinus angle
-		T ca = f.dotProduct ( t );	
+		T ca = f.dotProduct(t);
 
-		core::vector3df vt ( v * ( (T) 1 - ca ) );
+		core::vector3df vt(v * (1 - ca));
 
 		M[0] = vt.X * v.X + ca;
 		M[5] = vt.Y * v.Y + ca;
@@ -1844,84 +1961,78 @@ namespace core
 
 		M[1] = vt.X - vs.Z;
 		M[2] = vt.Z + vs.Y;
-		M[3] = (T) 0;
+		M[3] = 0;
 
 		M[4] = vt.X + vs.Z;
 		M[6] = vt.Y - vs.X;
-		M[7] = (T) 0;
+		M[7] = 0;
 
 		M[8] = vt.Z - vs.Y;
 		M[9] = vt.Y + vs.X;
-		M[11] = (T) 0;
+		M[11] = 0;
 
-		M[12] = (T) 0;
-		M[13] = (T) 0;
-		M[14] = (T) 0;
-		M[15] = (T) 1;
+		M[12] = 0;
+		M[13] = 0;
+		M[14] = 0;
+		M[15] = 1;
 
 		return *this;
 	}
 
 	//! Builds a matrix which rotates a source vector to a look vector over an arbitrary axis
-	/** \param camPos: viewer position in world coo
-	\param center: object position in world-coo and rotation pivot
+	/** \param camPos: viewer position in world coord
+	\param center: object position in world-coord, rotation pivot
 	\param translation: object final translation from center
 	\param axis: axis to rotate about
 	\param from: source vector to rotate from
 	 */
 	template <class T>
-	inline void CMatrix4<T>::buildAxisAlignedBillboard(	const core::vector3df& camPos,
-											const core::vector3df& center,
-											const core::vector3df& translation,
-											const core::vector3df& axis,
-											const core::vector3df& from
-										)
+	inline void CMatrix4<T>::buildAxisAlignedBillboard(
+				const core::vector3df& camPos,
+				const core::vector3df& center,
+				const core::vector3df& translation,
+				const core::vector3df& axis,
+				const core::vector3df& from)
 	{
 		// axis of rotation
 		core::vector3df up = axis;
-		up.normalize ();
-
-		core::vector3df forward = camPos - center;
-		forward.normalize();
-
-		core::vector3df right = up.crossProduct ( forward );
-		right.normalize ();
+		up.normalize();
+		const core::vector3df forward = (camPos - center).normalize();
+		const core::vector3df right = up.crossProduct(forward).normalize();
 
 		// correct look vector
-		core::vector3df look = right.crossProduct ( up );
+		const core::vector3df look = right.crossProduct(up);
 
 		// rotate from to
-
 		// axis multiplication by sin
-		core::vector3df vs = look.crossProduct ( from );
+		const core::vector3df vs = look.crossProduct(from);
 
 		// cosinus angle
-		f32 ca = from.dotProduct ( look );	
+		const f32 ca = from.dotProduct(look);
 
-		core::vector3df vt ( up * ( 1.f - ca ) );
+		core::vector3df vt(up * (1.f - ca));
 
-		M[0] = vt.X * up.X + ca;
-		M[5] = vt.Y * up.Y + ca;
-		M[10] = vt.Z * up.Z + ca;
+		M[0] = static_cast<T>(vt.X * up.X + ca);
+		M[5] = static_cast<T>(vt.Y * up.Y + ca);
+		M[10] = static_cast<T>(vt.Z * up.Z + ca);
 
 		vt.X *= up.Y;
 		vt.Z *= up.X;
 		vt.Y *= up.Z;
 
-		M[1] = vt.X - vs.Z;
-		M[2] = vt.Z + vs.Y;
-		M[3] = (T) 0;
+		M[1] = static_cast<T>(vt.X - vs.Z);
+		M[2] = static_cast<T>(vt.Z + vs.Y);
+		M[3] = 0;
 
-		M[4] = vt.X + vs.Z;
-		M[6] = vt.Y - vs.X;
-		M[7] = (T) 0;
+		M[4] = static_cast<T>(vt.X + vs.Z);
+		M[6] = static_cast<T>(vt.Y - vs.X);
+		M[7] = 0;
 
-		M[8] = vt.Z - vs.Y;
-		M[9] = vt.Y + vs.X;
-		M[11] = (T) 0;
-
-		setRotationCenter ( center, translation );
+		M[8] = static_cast<T>(vt.Z - vs.Y);
+		M[9] = static_cast<T>(vt.Y + vs.X);
+		M[11] = 0;
 
+		setRotationCenter(center, translation);
 	}
 
 
