@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2010 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -168,7 +168,7 @@ public:
 
 	virtual core::stringw getStringW()
 	{
-		return core::stringw(Value);
+		return core::stringw((double)Value);
 	}
 
 	virtual void setInt(s32 intValue)
@@ -496,6 +496,24 @@ public:
 		return v;
 	}
 
+	virtual core::vector2df getVector2d()
+	{
+		core::vector2df v;
+
+		if (IsFloat)
+		{
+			v.X = Count > 0 ? ValueF[0] : 0;
+			v.Y = Count > 1 ? ValueF[1] : 0;
+		}
+		else
+		{
+			v.X = (f32)(Count > 0 ? ValueI[0] : 0);
+			v.Y = (f32)(Count > 1 ? ValueI[1] : 0);
+		}
+
+		return v;
+	}
+
 	virtual video::SColorf getColorf()
 	{
 		video::SColorf c;
@@ -544,6 +562,23 @@ public:
 		return r;
 	}
 
+	virtual core::dimension2du getDimension2d()
+	{
+		core::dimension2d<u32> dim;
+
+		if (IsFloat)
+		{
+			dim.Width = (u32)(Count > 0 ? ValueF[0] : 0);
+			dim.Height = (u32)(Count > 1 ? ValueF[1] : 0);
+		}
+		else
+		{
+			dim.Width = (u32)(Count > 0 ? ValueI[0] : 0);
+			dim.Height = (u32)(Count > 1 ? ValueI[1] : 0);
+		}
+		return dim;
+	}
+
 	virtual core::matrix4 getMatrix()
 	{
 		core::matrix4 ret;
@@ -1076,8 +1111,8 @@ public:
 		}
 		else
 		{
-			if (Count > 0) ValueI[0] = v.Width;
-			if (Count > 1) ValueI[1] = v.Height;
+			if (Count > 0) ValueI[0] = (s32)v.Width;
+			if (Count > 1) ValueI[1] = (s32)v.Height;
 		}
 	}
 
@@ -1294,6 +1329,24 @@ public:
 	}
 };
 
+// Attribute implemented for 2d vectors
+class CVector2DAttribute : public CNumbersAttribute
+{
+public:
+
+	CVector2DAttribute(const char* name, core::vector2df value) : CNumbersAttribute(name, value) {}
+
+	virtual E_ATTRIBUTE_TYPE getType() const
+	{
+		return EAT_VECTOR2D;
+	}
+
+	virtual const wchar_t* getTypeString() const
+	{
+		return L"vector2d";
+	}
+};
+
 // Attribute implemented for 2d vectors
 class CPosition2DAttribute : public CNumbersAttribute
 {
@@ -1332,6 +1385,25 @@ public:
 	}
 };
 
+
+// Attribute implemented for dimension
+class CDimension2dAttribute : public CNumbersAttribute
+{
+public:
+
+	CDimension2dAttribute (const char* name, core::dimension2d<u32> value) : CNumbersAttribute(name, value) { }
+
+	virtual E_ATTRIBUTE_TYPE getType() const
+	{
+		return EAT_DIMENSION2D;
+	}
+
+	virtual const wchar_t* getTypeString() const
+	{
+		return L"dimension2d";
+	}
+};
+
 // Attribute implemented for matrices
 class CMatrixAttribute : public CNumbersAttribute
 {
@@ -1662,11 +1734,11 @@ public:
 	{
 		if (IsStringW)
 		{
-			ValueW = core::stringw(floatValue);
+			ValueW = core::stringw((double)floatValue);
 		}
 		else
 		{
-			Value = core::stringc(floatValue);
+			Value = core::stringc((double)floatValue);
 		}
 	};
 
@@ -1797,8 +1869,8 @@ class CTextureAttribute : public IAttribute
 {
 public:
 
-	CTextureAttribute(const char* name, video::ITexture* value, video::IVideoDriver* driver)
-		: Value(0), Driver(driver)
+	CTextureAttribute(const char* name, video::ITexture* value, video::IVideoDriver* driver, const io::path& filename)
+		: Value(0), Driver(driver), OverrideName(filename)
 	{
 		if (Driver)
 			Driver->grab();
@@ -1828,13 +1900,26 @@ public:
 
 	virtual core::stringw getStringW()
 	{
-		return core::stringw(Value ? Value->getName().getPath().c_str() : 0);
+		// (note: don't try to put all this in some ?: operators, or c++ builder will choke)
+		if ( OverrideName.size() )
+			return core::stringw(OverrideName);
+
+		if ( Value )
+			return core::stringw(Value->getName().getPath().c_str());
+
+		return core::stringw(0);
 	}
 
 	virtual core::stringc getString()
 	{
 		// since texture names can be stringw we are careful with the types
-		return core::stringc(Value ? Value->getName().getPath().c_str() : 0);
+		if ( OverrideName.size() )
+			return core::stringc(OverrideName);
+
+		if ( Value )
+			return core::stringc(Value->getName().getPath().c_str());
+		
+		return core::stringc(0);
 	}
 
 	virtual void setString(const char* text)
@@ -1842,7 +1927,10 @@ public:
 		if (Driver)
 		{
 			if (text && *text)
+			{
 				setTexture(Driver->getTexture(text));
+				OverrideName=text;
+			}
 			else
 				setTexture(0);
 		}
@@ -1875,6 +1963,7 @@ public:
 
 	video::ITexture* Value;
 	video::IVideoDriver* Driver;
+	io::path OverrideName;
 };
 
 
@@ -1945,7 +2034,9 @@ public:
 
 	virtual void setString(const char* text)
 	{
-		sscanf(text, "0x%x", (unsigned int*)(&Value));
+		u32 tmp;
+		sscanf(text, "0x%x", &tmp);
+		Value = (void *) tmp;
 	}
 
 	virtual E_ATTRIBUTE_TYPE getType() const
@@ -1973,6 +2064,7 @@ public:
 };
 
 
+
 // todo: CGUIFontAttribute
 
 } // end namespace io
